import { activity } from "maps";
import { system } from "@minecraft/server";

export function fillCone(player, dimension, baseCenter, height, baseRadius, blockTypeId, direction = "up", mode, fillMode = "solid") {
    const area = [];

    system.runTimeout(() => {
        for (let step = 0; step <= height; step++) {
            const dy = direction === "up" ? step : -step;
            const y = baseCenter.y + dy;
            const localRadius = baseRadius * (1 - (step / height));

            if (fillMode === "outline" && step === 0) continue;

            const minX = Math.round(baseCenter.x - localRadius);
            const maxX = Math.round(baseCenter.x + localRadius);
            const minZ = Math.round(baseCenter.z - localRadius);
            const maxZ = Math.round(baseCenter.z + localRadius);

            for (let x = minX; x <= maxX; x++) {
                for (let z = minZ; z <= maxZ; z++) {
                    const dx = x - baseCenter.x;
                    const dz = z - baseCenter.z;
                    const dist = Math.sqrt(dx * dx + dz * dz);

                    let shouldFill = false;

                    if (fillMode === "solid") {
                        shouldFill = dist <= localRadius + 0.5;
                    } else if (fillMode === "hollow") {
                        shouldFill = dist >= localRadius - 0.5 && dist <= localRadius + 0.5;
                    } else if (fillMode === "outline") {
                        const outlineThreshold = 0.3;
                        const onEdge = Math.abs(dist - localRadius) < outlineThreshold;
                        const isTop = step === height;
                        shouldFill = onEdge || isTop;
                    }

                    if (!shouldFill) continue;

                    const loc = { x, y, z };
                    const block = dimension.getBlock(loc);
                    if (!block) continue;

                    area.push({
                        block: block.typeId,
                        location: loc
                    });

                    const isAir = block.typeId === "minecraft:air";

                    if (mode === "build") {
                        block.setType(blockTypeId);
                    } else if (mode === "destruct" && !isAir) {
                        block.setType("minecraft:air");
                    }
                }
            }
        }
    }, 2);

    const act = [];
    act.push(area);

    if (!activity.has(player.id)) {
        activity.set(player.id, act);
    } else {
        const data = activity.get(player.id);
        data.push(act);
        activity.set(player.id, data);
    }
}

export function fillSphere(player, dimension, center, radius, blockTypeId, mode, fillMode = "solid") {
    const area = [];

    const minX = Math.floor(center.x - radius);
    const maxX = Math.floor(center.x + radius);
    const minY = Math.floor(center.y - radius);
    const maxY = Math.floor(center.y + radius);
    const minZ = Math.floor(center.z - radius);
    const maxZ = Math.floor(center.z + radius);

    system.runTimeout(() => {
        for (let x = minX; x <= maxX; x++) {
            for (let y = minY; y <= maxY; y++) {
                for (let z = minZ; z <= maxZ; z++) {
                    const dx = x - center.x;
                    const dy = y - center.y;
                    const dz = z - center.z;
                    const distance = Math.sqrt(dx * dx + dy * dy + dz * dz);

                    let shouldFill = false;

                    if (fillMode === "solid") {
                        shouldFill = distance <= radius + 0.5;
                    } else if (fillMode === "hollow") {
                        shouldFill = distance >= radius - 0.5 && distance <= radius + 0.5;
                    } else if (fillMode === "outline") {
                        shouldFill = Math.abs(distance - radius) < 0.3;
                    }

                    if (!shouldFill) continue;

                    const loc = { x, y, z };
                    const block = dimension.getBlock(loc);
                    if (!block) continue;

                    area.push({
                        block: block.typeId,
                        location: loc
                    });

                    const isAir = block.typeId === "minecraft:air";

                    if (mode === "build") {
                        block.setType(blockTypeId);
                    } else if (mode === "destruct" && !isAir) {
                        block.setType("minecraft:air");
                    }
                }
            }
        }
    }, 2);

    const act = [];
    act.push(area);

    if (!activity.has(player.id)) {
        activity.set(player.id, act);
    } else {
        const data = activity.get(player.id);
        data.push(act);
        activity.set(player.id, data);
    }
}

export function fillCube(player, dimension, startLoc, endLoc, blockTypeId, mode, fillMode = "solid") {
    const area = [];

    const minX = Math.min(startLoc.x, endLoc.x);
    const maxX = Math.max(startLoc.x, endLoc.x);
    const minY = Math.min(startLoc.y, endLoc.y);
    const maxY = Math.max(startLoc.y, endLoc.y);
    const minZ = Math.min(startLoc.z, endLoc.z);
    const maxZ = Math.max(startLoc.z, endLoc.z);

    system.runTimeout(() => {
        for (let x = minX; x <= maxX; x++) {
            for (let y = minY; y <= maxY; y++) {
                for (let z = minZ; z <= maxZ; z++) {
                    const isEdge =
                        x === minX || x === maxX ||
                        y === minY || y === maxY ||
                        z === minZ || z === maxZ;

                    const isOutline =
                        (x === minX || x === maxX) + (y === minY || y === maxY) + (z === minZ || z === maxZ) >= 2;

                    if (
                        (fillMode === "hollow" && !isEdge) ||
                        (fillMode === "outline" && !isOutline)
                    ) {
                        continue;
                    }

                    const loc = { x, y, z };
                    const block = dimension.getBlock(loc);
                    if (!block) continue;

                    area.push({
                        block: block.typeId,
                        location: loc
                    });

                    const isAir = block.typeId === "minecraft:air";

                    if (mode === "build") {
                        block.setType(blockTypeId);
                    } else if (mode === "destruct" && !isAir) {
                        block.setType("minecraft:air");
                    }
                }
            }
        }
    }, 2);

    const act = [];
    act.push(area);

    if (!activity.has(player.id)) {
        activity.set(player.id, act);
    } else {
        const data = activity.get(player.id);
        data.push(act);
        activity.set(player.id, data);
    }
}